@@ -26,6 +26,10 @@
  *    Rob Clark <robclark@freedesktop.org>
  */
 
+#ifdef HAVE_CONFIG_H
+# include <config.h>
+#endif
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <unistd.h>
@@ -34,7 +38,6 @@
 #include "freedreno_priv.h"
 
 static pthread_mutex_t table_lock = PTHREAD_MUTEX_INITIALIZER;
-static void * dev_table;
 
 struct fd_device * kgsl_device_new(int fd);
 struct fd_device * msm_device_new(int fd);
@@ -77,7 +80,7 @@ init_cache_buckets(struct fd_device *dev)
 	}
 }
 
-static struct fd_device * fd_device_new_impl(int fd)
+struct fd_device * fd_device_new(int fd)
 {
 	struct fd_device *dev;
 	drmVersionPtr version;
@@ -89,16 +92,19 @@ static struct fd_device * fd_device_new_impl(int fd)
 		return NULL;
 	}
 
-	if (!strcmp(version->name, "kgsl")) {
-		DEBUG_MSG("kgsl DRM device");
-		dev = kgsl_device_new(fd);
-	} else if (!strcmp(version->name, "msm")) {
+	if (!strcmp(version->name, "msm")) {
 		DEBUG_MSG("msm DRM device");
 		dev = msm_device_new(fd);
+#ifdef HAVE_FREEDRENO_KGSL
+	} else if (!strcmp(version->name, "kgsl")) {
+		DEBUG_MSG("kgsl DRM device");
+		dev = kgsl_device_new(fd);
+#endif
 	} else {
 		ERROR_MSG("unknown device: %s", version->name);
 		dev = NULL;
 	}
+	drmFreeVersion(version);
 
 	if (!dev)
 		return NULL;
@@ -112,26 +118,14 @@ static struct fd_device * fd_device_new_impl(int fd)
 	return dev;
 }
 
-struct fd_device * fd_device_new(int fd)
+/* like fd_device_new() but creates it's own private dup() of the fd
+ * which is close()d when the device is finalized.
+ */
+struct fd_device * fd_device_new_dup(int fd)
 {
-	struct fd_device *dev = NULL;
-	int key = fd;
-
-	pthread_mutex_lock(&table_lock);
-
-	if (!dev_table)
-		dev_table = drmHashCreate();
-
-	if (drmHashLookup(dev_table, key, (void **)&dev)) {
-		dev = fd_device_new_impl(fd);
-		if (dev)
-			drmHashInsert(dev_table, key, dev);
-	} else {
-		dev = fd_device_ref(dev);
-	}
-
-	pthread_mutex_unlock(&table_lock);
-
+	struct fd_device *dev = fd_device_new(dup(fd));
+	if (dev)
+		dev->closefd = 1;
 	return dev;
 }
 
@@ -146,11 +140,12 @@ static void fd_device_del_impl(struct fd_device *dev)
 	fd_cleanup_bo_cache(dev, 0);
 	drmHashDestroy(dev->handle_table);
 	drmHashDestroy(dev->name_table);
-	drmHashDelete(dev_table, dev->fd);
+	if (dev->closefd)
+		close(dev->fd);
 	dev->funcs->destroy(dev);
 }
 
-void fd_device_del_locked(struct fd_device *dev)
+drm_private void fd_device_del_locked(struct fd_device *dev)
 {
 	if (!atomic_dec_and_test(&dev->refcnt))
 		return;
@@ -165,3 +160,8 @@ void fd_device_del(struct fd_device *dev)
 	fd_device_del_impl(dev);
 	pthread_mutex_unlock(&table_lock);
 }
+
+int fd_device_fd(struct fd_device *dev)
+{
+	return dev->fd;
+}
